@@ -315,6 +315,9 @@ on_timeline_completed_cb (GcalTimeline            *timeline,
 
   GCAL_ENTRY;
 
+  if (!self->pending_search)
+    GCAL_RETURN ();
+
   if (!gcal_timeline_is_complete (timeline))
     GCAL_RETURN ();
 
@@ -407,6 +410,7 @@ gcal_shell_search_provider_add_event (GcalTimelineSubscriber *subscriber,
 
 static void
 gcal_shell_search_provider_update_event (GcalTimelineSubscriber *subscriber,
+                                         GcalEvent              *old_event,
                                          GcalEvent              *event)
 {
 }
