@@ -20,6 +20,8 @@
 
 #define G_LOG_DOMAIN "GcalNewCalendarPage"
 
+#include "config.h"
+
 #include <glib/gi18n.h>
 
 #include "gcal-context.h"
@@ -42,20 +44,17 @@ typedef enum
 
 struct _GcalNewCalendarPage
 {
-  GtkBox              parent;
+  AdwBin              parent;
 
   GtkWidget          *add_button;
   GtkEntry           *calendar_address_entry;
   EntryState          calendar_address_entry_state;
   GcalFileChooserButton *calendar_file_chooser_button;
-  GtkWidget          *cancel_button;
-  GtkWidget          *credentials_cancel_button;
-  GtkWidget          *credentials_connect_button;
-  GtkWindow          *credentials_dialog;
+  AdwMessageDialog   *credentials_dialog;
   GtkEntry           *credentials_password_entry;
   GtkEntry           *credentials_user_entry;
   GtkColorDialogButton *local_calendar_color_button;
-  GtkEntry           *local_calendar_name_entry;
+  AdwEntryRow        *local_calendar_name_row;
   GtkWidget          *web_sources_listbox;
   GtkWidget          *web_sources_revealer;
 
@@ -78,7 +77,7 @@ static void          sources_discovered_cb                       (GObject
 
 static void          gcal_calendar_management_page_iface_init    (GcalCalendarManagementPageInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (GcalNewCalendarPage, gcal_new_calendar_page, GTK_TYPE_BOX,
+G_DEFINE_TYPE_WITH_CODE (GcalNewCalendarPage, gcal_new_calendar_page, ADW_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_CALENDAR_MANAGEMENT_PAGE,
                                                 gcal_calendar_management_page_iface_init))
 
@@ -137,13 +136,36 @@ calendar_path_to_name_suggestion (GFile *file)
 static void
 update_add_button (GcalNewCalendarPage *self)
 {
+  g_autofree gchar *add_button_label = NULL;
   gboolean valid;
+  uint32_t n_calendars;
 
   valid = (self->local_source != NULL || self->remote_sources != NULL) &&
       self->calendar_address_entry_state != ENTRY_STATE_VALIDATING &&
       self->calendar_address_entry_state != ENTRY_STATE_INVALID;
 
   gtk_widget_set_sensitive (self->add_button, valid);
+
+  n_calendars = 0;
+  if (self->local_source)
+    n_calendars++;
+  if (self->remote_sources)
+    n_calendars += self->remote_sources->len;
+
+  if (n_calendars > 0)
+    {
+      add_button_label = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
+                                                       "Add Calendar",
+                                                       "Add %1$u Calendars",
+                                                       n_calendars),
+                                          n_calendars);
+    }
+  else
+    {
+      add_button_label = g_strdup (_("Add Calendar"));
+    }
+
+  gtk_button_set_label (GTK_BUTTON (self->add_button), add_button_label);
 }
 
 static void
@@ -153,7 +175,7 @@ update_local_source (GcalNewCalendarPage *self)
 
   g_clear_object (&self->local_source);
 
-  calendar_name = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->local_calendar_name_entry)));
+  calendar_name = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->local_calendar_name_row)));
   calendar_name = g_strstrip (calendar_name);
 
   if (calendar_name && g_utf8_strlen (calendar_name, -1) > 0)
@@ -236,15 +258,6 @@ discover_sources (GcalNewCalendarPage *self)
   GCAL_EXIT;
 }
 
-static void
-hide_credentials_dialog (GcalNewCalendarPage *self)
-{
-  gtk_editable_set_text (GTK_EDITABLE (self->credentials_user_entry), "");
-  gtk_editable_set_text (GTK_EDITABLE (self->credentials_password_entry), "");
-
-  gtk_window_close (self->credentials_dialog);
-}
-
 /*
  * Callbacks
  */
@@ -326,8 +339,8 @@ sources_discovered_cb (GObject      *source_object,
           g_debug ("Unauthorized, asking for credentials...");
 
           root = gtk_widget_get_root (GTK_WIDGET (self));
-          gtk_window_set_transient_for (self->credentials_dialog, GTK_WINDOW (root));
-          gtk_window_present (self->credentials_dialog);
+          gtk_window_set_transient_for (GTK_WINDOW (self->credentials_dialog), GTK_WINDOW (root));
+          gtk_window_present (GTK_WINDOW (self->credentials_dialog));
         }
       else
         {
@@ -406,23 +419,17 @@ on_calendar_address_activated_cb (GtkEntry            *entry,
 }
 
 static void
-on_credential_button_clicked_cb (GtkWidget           *button,
+on_credentials_dialog_response_cb (AdwMessageDialog    *dialog,
+                                 const gchar         *response,
                                  GcalNewCalendarPage *self)
 {
-  if (button == self->credentials_connect_button)
+  if (g_str_equal (response, "connect"))
     discover_sources (self);
-  else
+  else if (g_str_equal (response, "cancel"))
     update_url_entry_state (self, ENTRY_STATE_INVALID);
 
-  hide_credentials_dialog (self);
-}
-
-static void
-on_credential_entry_activate_cb (GtkEntry            *entry,
-                                 GcalNewCalendarPage *self)
-{
-  discover_sources (self);
-  hide_credentials_dialog (self);
+  gtk_editable_set_text (GTK_EDITABLE (self->credentials_user_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->credentials_password_entry), "");
 }
 
 static void
@@ -467,16 +474,9 @@ on_url_entry_text_changed_cb (GtkEntry            *entry,
 }
 
 static void
-on_cancel_button_clicked_cb (GtkWidget                  *button,
-                             GcalCalendarManagementPage *page)
-{
-  gcal_calendar_management_page_switch_page (page, "calendars", NULL);
-}
-
-static void
-on_local_calendar_name_entry_text_changed_cb (GtkEntry            *entry,
-                                              GParamSpec          *pspec,
-                                              GcalNewCalendarPage *self)
+on_local_calendar_name_row_text_changed_cb (AdwEntryRow         *entry_row,
+                                            GParamSpec          *pspec,
+                                            GcalNewCalendarPage *self)
 {
   update_local_source (self);
 }
@@ -496,7 +496,7 @@ on_web_description_label_link_activated_cb (GtkLabel            *label,
 {
   GDBusConnection *connection = g_application_get_dbus_connection (g_application_get_default ());
 
-  gcal_utils_launch_online_accounts_panel (connection, NULL, NULL);
+  gcal_utils_launch_gnome_settings (connection, "online-accounts", NULL);
 }
 
 
@@ -516,44 +516,20 @@ gcal_new_calendar_page_get_title (GcalCalendarManagementPage *page)
   return _("New Calendar");
 }
 
-static void
-gcal_new_calendar_page_activate (GcalCalendarManagementPage *page,
-                                 GcalCalendar               *calendar)
-{
-  GcalNewCalendarPage *self;
-  GtkHeaderBar *headerbar;
-
-  GCAL_ENTRY;
-
-  self = GCAL_NEW_CALENDAR_PAGE (page);
-  headerbar = gcal_calendar_management_page_get_titlebar (page);
-
-  gtk_header_bar_pack_start (headerbar, self->cancel_button);
-  gtk_header_bar_pack_end (headerbar, self->add_button);
-  gtk_header_bar_set_show_title_buttons (headerbar, FALSE);
-
-  GCAL_EXIT;
-}
 static void
 gcal_new_calendar_page_deactivate (GcalCalendarManagementPage *page)
 {
   GcalNewCalendarPage *self;
-  GtkHeaderBar *headerbar;
 
   GCAL_ENTRY;
 
   self = GCAL_NEW_CALENDAR_PAGE (page);
-  headerbar = gcal_calendar_management_page_get_titlebar (page);
-
-  gtk_header_bar_remove (headerbar, self->cancel_button);
-  gtk_header_bar_remove (headerbar, self->add_button);
-  gtk_header_bar_set_show_title_buttons (headerbar, TRUE);
 
   g_clear_object (&self->local_source);
   g_clear_pointer (&self->remote_sources, g_ptr_array_unref);
   update_add_button (self);
 
-  gtk_editable_set_text (GTK_EDITABLE (self->local_calendar_name_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->local_calendar_name_row), "");
   gtk_editable_set_text (GTK_EDITABLE (self->calendar_address_entry), "");
 
   toggle_url_entry_pulsing (self, FALSE);
@@ -566,7 +542,6 @@ gcal_calendar_management_page_iface_init (GcalCalendarManagementPageInterface *i
 {
   iface->get_name = gcal_new_calendar_page_get_name;
   iface->get_title = gcal_new_calendar_page_get_title;
-  iface->activate = gcal_new_calendar_page_activate;
   iface->deactivate = gcal_new_calendar_page_deactivate;
 }
 
@@ -658,24 +633,19 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, add_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_address_entry);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_file_chooser_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, cancel_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_cancel_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_connect_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_dialog);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_password_entry);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_user_entry);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, local_calendar_color_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, local_calendar_name_entry);
+  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, local_calendar_name_row);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, web_sources_listbox);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, web_sources_revealer);
 
   gtk_widget_class_bind_template_callback (widget_class, on_add_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_calendar_address_activated_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_cancel_button_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_credential_button_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_credential_entry_activate_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_credentials_dialog_response_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_file_chooser_button_file_changed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_name_entry_text_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_name_row_text_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_color_button_rgba_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_url_entry_text_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_web_description_label_link_activated_cb);
