@@ -236,7 +236,7 @@ gcal_application_show_about (GSimpleAction *simple,
                         "version", VERSION,
                         "copyright", copyright,
                         "website", "https://apps.gnome.org/Calendar/",
-                        "issue-url", "https://gitlab.gnome.org/GNOME/gnome-calendar/-/issues/new",
+                        "issue-url", "https://gitlab.gnome.org/GNOME/gnome-calendar/-/issues",
                         "support-url", "https://discourse.gnome.org/tag/calendar",
                         "license-type", GTK_LICENSE_GPL_3_0,
                         "developers", developers,
