@@ -33,7 +33,7 @@ typedef enum
 } GcalEventPreviewAction;
 
 #define GCAL_TYPE_EVENT_WIDGET                    (gcal_event_widget_get_type ())
-G_DECLARE_FINAL_TYPE (GcalEventWidget, gcal_event_widget, GCAL, EVENT_WIDGET, GtkBin)
+G_DECLARE_FINAL_TYPE (GcalEventWidget, gcal_event_widget, GCAL, EVENT_WIDGET, GtkWidget)
 
 typedef void         (*GcalEventPreviewCallback)                 (GcalEventWidget        *event_widget,
                                                                   GcalEventPreviewAction  action,
@@ -57,6 +57,9 @@ void                 gcal_event_widget_set_date_end              (GcalEventWidge
 void                 gcal_event_widget_set_read_only             (GcalEventWidget    *event,
                                                                   gboolean            read_only);
 
+void                 gcal_event_widget_set_timestamp_policy      (GcalEventWidget     *event,
+                                                                  GcalTimestampPolicy  timestamp_policy);
+
 void                 gcal_event_widget_show_preview              (GcalEventWidget          *self,
                                                                   GcalEventPreviewCallback  callback,
                                                                   gpointer                  user_data);
