@@ -20,12 +20,14 @@
 
 #pragma once
 
+#include <adwaita.h>
+
 #include "gcal-context.h"
 
 G_BEGIN_DECLS
 
 #define GCAL_TYPE_IMPORT_DIALOG (gcal_import_dialog_get_type())
-G_DECLARE_FINAL_TYPE (GcalImportDialog, gcal_import_dialog, GCAL, IMPORT_DIALOG, GtkDialog)
+G_DECLARE_FINAL_TYPE (GcalImportDialog, gcal_import_dialog, GCAL, IMPORT_DIALOG, AdwWindow)
 
 GtkWidget*           gcal_import_dialog_new_for_files            (GcalContext        *context,
                                                                   GFile             **files,
