@@ -97,6 +97,26 @@ update_style_flags (GcalMonthCell *self)
     gtk_widget_remove_css_class (GTK_WIDGET (self), "workday");
 }
 
+static void
+add_month_separators (GcalMonthCell *self)
+{
+  gint day_of_month;
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "separator-top");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "separator-side");
+
+  day_of_month = g_date_time_get_day_of_month (self->date);
+  if (day_of_month > 1 && day_of_month <= 7)
+    {
+      gtk_widget_add_css_class (GTK_WIDGET (self), "separator-top");
+    }
+  else if (day_of_month == 1)
+    {
+      gtk_widget_add_css_class (GTK_WIDGET (self), "separator-side");
+      gtk_widget_add_css_class (GTK_WIDGET (self), "separator-top");
+    }
+}
+
 static void
 move_event (GcalMonthCell         *self,
             GcalEvent             *event,
@@ -293,6 +313,9 @@ gcal_month_cell_finalize (GObject *object)
 {
   GcalMonthCell *self = (GcalMonthCell *)object;
 
+  GcalWeatherService *weather_service = gcal_context_get_weather_service (self->context);
+  gcal_weather_service_release (weather_service);
+
   gcal_clear_date_time (&self->date);
   g_clear_object (&self->context);
 
@@ -430,6 +453,7 @@ gcal_month_cell_set_date (GcalMonthCell *self,
 
   gtk_label_set_text (self->day_label, text);
   update_style_flags (self);
+  add_month_separators (self);
 
   /* Month name */
   day_of_month = g_date_time_get_day_of_month (date);
@@ -497,6 +521,9 @@ gcal_month_cell_set_context (GcalMonthCell *self,
                            self,
                            0);
 
+  GcalWeatherService *weather_service = gcal_context_get_weather_service (self->context);
+  gcal_weather_service_hold (weather_service);
+
   update_weather (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CONTEXT]);
