@@ -94,9 +94,6 @@ gsize                e_utf8_strftime_fix_am_pm                   (gchar
                                                                   const gchar        *fmt,
                                                                   const struct tm    *tm);
 
-void                 fix_popover_menu_icons                      (GtkPopover         *popover);
-
-
 void                 get_source_parent_name_color                (GcalManager        *manager,
                                                                   ESource            *source,
                                                                   gchar             **name,
@@ -136,4 +133,8 @@ void                 gcal_utils_ask_recurrence_modification_type (GtkWidget
 
 const gchar *        gcal_util_translate_time_string             (const gchar          *str);
 
+GTimeZone *          gcal_util_get_app_timezone_or_local         ();
+
+gboolean             gcal_is_valid_event_name                    (const gchar          *event_name);
+
 #endif /* __GCAL_UTILS_H__ */
