@@ -49,11 +49,13 @@ typedef struct
 /* GcalWeatherService:
  *
  * @time_zone:               The current time zone
+ * @use_counter:             Number of active users of the service.
  * @check_interval_new:      Amount of seconds to wait before fetching weather infos.
  * @check_interval_renew:    Amount of seconds to wait before re-fetching weather information.
  * @duration_timer           Timer used to request weather report updates.
  * @midnight_timer           Timer used to update weather reports at midnight.
  * @network_changed_sid      "network-changed" signal ID.
+ * @location:                Used for from where to display the weather. Current location if NULL.
  * @location_service:        Used to monitor location changes.
  *                           Initialized by gcal_weather_service_run(),
  *                           freed by gcal_weather_service_stop().
@@ -64,7 +66,9 @@ typedef struct
  * @valid_timespan:          Amount of seconds weather information are considered valid.
  * @gweather_info:           The weather info to query.
  * @max_days:                Number of days we want weather information for.
+ * @weather_service_active:  True if weather service is requested to be active, regardless of being in use.
  * @weather_service_running: True if weather service is active.
+ * @weather_is_stale:        True if update was request without the service running.
  *
  * This service listens to location and weather changes and reports them.
  *
@@ -81,6 +85,8 @@ struct _GcalWeatherService
 
   GTimeZone          *timezone;            /* owned, nullable */
 
+  guint               use_counter;
+
   /* timer: */
   guint               check_interval_new;
   guint               check_interval_renew;
@@ -91,6 +97,7 @@ struct _GcalWeatherService
   gulong              network_changed_sid;
 
   /* locations: */
+  GWeatherLocation   *location;             /* owned, nullable */
   GClueSimple        *location_service;     /* owned, nullable */
   GCancellable       *location_cancellable; /* owned, non-null */
   gboolean            location_service_running;
@@ -101,7 +108,9 @@ struct _GcalWeatherService
   gint64              valid_timespan;
   GWeatherInfo       *gweather_info;        /* owned, nullable */
   guint               max_days;
+  gboolean            weather_service_active;
   gboolean            weather_service_running;
+  gboolean            weather_is_stale;
 };
 
 static void          on_gweather_update_cb                       (GWeatherInfo       *info,
@@ -113,6 +122,7 @@ enum
 {
   PROP_0,
   PROP_TIME_ZONE,
+  PROP_LOCATION,
   PROP_NUM,
 };
 
@@ -476,8 +486,8 @@ preprocess_gweather_reports (GcalWeatherService *self,
   GSList *iter = NULL;    /* unowned */
   GDate cur_gdate;
   glong first_tomorrow_dtime = -1;
-  glong today_unix;
-  glong unix_now;
+  gint64 today_unix;
+  gint64 unix_now;
   guint i;
 
   /*
@@ -683,6 +693,15 @@ update_gclue_location (GcalWeatherService  *self,
   update_location (self, wlocation);
 }
 
+static void
+start_or_stop_weather_service (GcalWeatherService *self)
+{
+  if (self->use_counter > 0 && self->weather_service_active)
+    gcal_weather_service_start (self);
+  else
+    gcal_weather_service_stop (self);
+}
+
 
 /*
  * Callbacks
@@ -887,6 +906,10 @@ gcal_weather_service_get_property (GObject    *object,
       g_value_set_pointer (value, gcal_weather_service_get_time_zone (self));
       break;
 
+    case PROP_LOCATION:
+      g_value_set_pointer (value, gcal_weather_service_get_location (self));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -907,6 +930,10 @@ gcal_weather_service_set_property (GObject      *object,
       gcal_weather_service_set_time_zone (self, g_value_get_pointer (value));
       break;
 
+    case PROP_LOCATION:
+      gcal_weather_service_set_location (self, g_value_get_pointer (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -935,6 +962,19 @@ gcal_weather_service_class_init (GcalWeatherServiceClass *klass)
                                                          "time-zone",
                                                          G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
 
+  /**
+   * GcalWeatherService:location:
+   *
+   * The location to use, automatic if NULL.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_LOCATION,
+                                   g_param_spec_object ("location",
+                                                        "location",
+                                                        "location",
+                                                        GWEATHER_TYPE_LOCATION,
+                                                        G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
   /**
    * GcalWeatherService::weather-changed:
    * @sender: The #GcalWeatherService
@@ -991,6 +1031,43 @@ gcal_weather_service_new (void)
   return g_object_new (GCAL_TYPE_WEATHER_SERVICE, NULL);
 }
 
+/**
+ * gcal_weather_service_get_location:
+ * @self: The #GcalWeatherService instance.
+ *
+ * Getter for #GcalWeatherService:location.
+ */
+GWeatherLocation*
+gcal_weather_service_get_location (GcalWeatherService *self)
+{
+  g_return_val_if_fail (GCAL_IS_WEATHER_SERVICE (self), NULL);
+
+  return self->location;
+}
+
+/**
+ * gcal_weather_service_set_location:
+ * @self: The #GcalWeatherService instance.
+ * @value: new location, nullable.
+ *
+ * Setter for #GcalWeatherInfos:location.
+ */
+void
+gcal_weather_service_set_location (GcalWeatherService *self,
+                                   GWeatherLocation   *value)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
+
+  if (self->location == value)
+    return;
+
+  self->location = value;
+
+  gcal_weather_service_start (self);
+
+  g_object_notify (G_OBJECT (self), "location");
+}
+
 /**
  * gcal_weather_service_get_time_zone:
  * @self: The #GcalWeatherService instance.
@@ -1108,6 +1185,16 @@ gcal_weather_service_update (GcalWeatherService *self)
 {
   g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
 
+  if (!self->weather_service_running)
+    {
+      self->weather_is_stale = TRUE;
+      return;
+    }
+  else
+    {
+      self->weather_is_stale = FALSE;
+    }
+
   if (self->gweather_info)
     {
       gweather_info_update (self->gweather_info);
@@ -1118,17 +1205,49 @@ gcal_weather_service_update (GcalWeatherService *self)
     }
 }
 
+
 /**
- * gcal_weather_service_run:
+ * gcal_weather_service_activate:
+ * @self: The #GcalWeatherService instance.
+ *
+ * Activates weather service if needed.
+ * Use ::weather-changed to catch responses.
+ */
+void
+gcal_weather_service_activate (GcalWeatherService *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
+
+  self->weather_service_active = TRUE;
+
+  start_or_stop_weather_service (self);
+}
+
+/**
+ * gcal_weather_service_deactivate:
+ * @self: The #GcalWeatherService instance.
+ *
+ * Stops the service.
+ */
+void
+gcal_weather_service_deactivate (GcalWeatherService *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
+
+  self->weather_service_active = FALSE;
+
+  start_or_stop_weather_service (self);
+}
+
+/**
+ * gcal_weather_service_start:
  * @self: The #GcalWeatherService instance.
- * @location: (nullable): A fixed location or %NULL to use Gclue.
  *
  * Starts to monitor location and weather changes.
  * Use ::weather-changed to catch responses.
  */
 void
-gcal_weather_service_run (GcalWeatherService *self,
-                          GWeatherLocation   *location)
+gcal_weather_service_start (GcalWeatherService *self)
 {
   g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
 
@@ -1139,7 +1258,7 @@ gcal_weather_service_run (GcalWeatherService *self,
 
   self->weather_service_running = TRUE;
 
-  if (!location)
+  if (!self->location)
     {
       /* Start location and weather service: */
       self->location_service_running = TRUE;
@@ -1157,8 +1276,15 @@ gcal_weather_service_run (GcalWeatherService *self,
     {
       self->location_service_running = FALSE;
 
+      /* TODO: stop running location service */
+
       /*_update_location starts timer if necessary */
-      update_location (self, location);
+      update_location (self, self->location);
+    }
+
+  if (self->weather_is_stale)
+    {
+      gcal_weather_service_update (self);
     }
 }
 
@@ -1203,8 +1329,43 @@ gcal_weather_service_stop (GcalWeatherService *self)
                                   (GAsyncReadyCallback) on_gclue_client_stopped_cb,
                                   self->location_service);
         }
-      self->location_service = NULL;
+      g_clear_object (&self->location_service);
     }
 
   GCAL_EXIT;
 }
+
+/**
+ * gcal_weather_service_hold:
+ * @self: The #GcalWeatherService instance.
+ *
+ * Mark the service as in use by increasing its use counter.
+ **/
+void
+gcal_weather_service_hold (GcalWeatherService *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
+
+  self->use_counter++;
+
+  start_or_stop_weather_service (self);
+}
+
+/**
+ * gcal_weather_service_release:
+ * @self: The #GcalWeatherService instance.
+ *
+ * Mark the use the of service to be over by decreasing its use counter.
+ * A call to this function is expected to be paired with a previous
+ * call to gcal_weather_service_hold.
+ **/
+void
+gcal_weather_service_release (GcalWeatherService *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
+  g_return_if_fail (self->use_counter > 0);
+
+  self->use_counter--;
+
+  start_or_stop_weather_service (self);
+}
